import { __rest, __spreadArray } from "tslib";
import React from 'react';
import { autocompletion, closeBrackets, closeBracketsKeymap, completionKeymap } from '@codemirror/autocomplete';
import { defaultKeymap, history, historyKeymap } from '@codemirror/commands';
import { markdown } from '@codemirror/lang-markdown';
import { bracketMatching, defaultHighlightStyle, foldGutter, foldKeymap, indentOnInput, syntaxHighlighting } from '@codemirror/language';
import { lintKeymap } from '@codemirror/lint';
import { highlightSelectionMatches, searchKeymap } from '@codemirror/search';
import { EditorState } from '@codemirror/state';
import { crosshairCursor, drawSelection, dropCursor, EditorView, highlightActiveLine, highlightActiveLineGutter, highlightSpecialChars, keymap, lineNumbers, rectangularSelection } from '@codemirror/view';
import className from './style.module.css';
var basicSetup = [
    lineNumbers(),
    highlightActiveLineGutter(),
    highlightSpecialChars(),
    history(),
    foldGutter(),
    drawSelection(),
    dropCursor(),
    EditorState.allowMultipleSelections.of(true),
    indentOnInput(),
    syntaxHighlighting(defaultHighlightStyle, { fallback: true }),
    bracketMatching(),
    closeBrackets(),
    autocompletion(),
    rectangularSelection(),
    crosshairCursor(),
    highlightActiveLine(),
    highlightSelectionMatches(),
    keymap.of(__spreadArray(__spreadArray(__spreadArray(__spreadArray(__spreadArray(__spreadArray(__spreadArray([], closeBracketsKeymap, true), defaultKeymap, true), searchKeymap, true), historyKeymap, true), foldKeymap, true), completionKeymap, true), lintKeymap, true))
];
var createCodeMirrorState = function (_a) {
    var onChange = _a.onChange, lock = _a.lock, dark = _a.dark, editable = _a.editable, value = _a.value;
    return EditorState.create({
        doc: value,
        extensions: [
            basicSetup,
            markdown(),
            EditorView.updateListener.of(function (v) {
                if (v.focusChanged) {
                    lock.current = v.view.hasFocus;
                }
                if (v.docChanged) {
                    var getString = function () { return v.state.doc.toString(); };
                    onChange(getString);
                }
            }),
            EditorView.theme({
                '&.cm-focused': {
                    outline: 'none'
                }
            }, { dark: dark }),
            // https://github.com/codemirror/codemirror.next/issues/173
            EditorView.editable.of(editable)
        ]
    });
};
var createCodeMirrorView = function (_a) {
    var root = _a.root, options = __rest(_a, ["root"]);
    return new EditorView({
        state: createCodeMirrorState(options),
        parent: root
    });
};
export var CodeMirror = React.forwardRef(function (_a, ref) {
    var value = _a.value, onChange = _a.onChange, lock = _a.lock, dark = _a.dark, editable = _a.editable;
    var divRef = React.useRef(null);
    var editorRef = React.useRef();
    var _b = React.useState(false), focus = _b[0], setFocus = _b[1];
    React.useEffect(function () {
        if (!divRef.current)
            return;
        var editor = createCodeMirrorView({
            root: divRef.current,
            onChange: onChange,
            lock: lock,
            dark: dark,
            editable: editable,
            value: value
        });
        editorRef.current = editor;
        return function () {
            editor.destroy();
        };
    }, [onChange, value, lock, dark]);
    React.useImperativeHandle(ref, function () { return ({
        update: function (value) {
            var current = editorRef.current;
            if (!current)
                return;
            current.setState(createCodeMirrorState({ onChange: onChange, lock: lock, dark: dark, editable: editable, value: value }));
        }
    }); });
    return (React.createElement("div", { onFocus: function () { return setFocus(true); }, onBlur: function () { return setFocus(false); }, className: className['code'] + (focus ? ' ' + className['focus'] : '') },
        React.createElement("div", { ref: divRef })));
});
//# sourceMappingURL=CodeMirror.js.map