"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CodeMirror = void 0;
var tslib_1 = require("tslib");
var react_1 = tslib_1.__importDefault(require("react"));
var autocomplete_1 = require("@codemirror/autocomplete");
var commands_1 = require("@codemirror/commands");
var lang_markdown_1 = require("@codemirror/lang-markdown");
var language_1 = require("@codemirror/language");
var lint_1 = require("@codemirror/lint");
var search_1 = require("@codemirror/search");
var state_1 = require("@codemirror/state");
var view_1 = require("@codemirror/view");
var style_module_css_1 = tslib_1.__importDefault(require("./style.module.css"));
var basicSetup = [
    (0, view_1.lineNumbers)(),
    (0, view_1.highlightActiveLineGutter)(),
    (0, view_1.highlightSpecialChars)(),
    (0, commands_1.history)(),
    (0, language_1.foldGutter)(),
    (0, view_1.drawSelection)(),
    (0, view_1.dropCursor)(),
    state_1.EditorState.allowMultipleSelections.of(true),
    (0, language_1.indentOnInput)(),
    (0, language_1.syntaxHighlighting)(language_1.defaultHighlightStyle, { fallback: true }),
    (0, language_1.bracketMatching)(),
    (0, autocomplete_1.closeBrackets)(),
    (0, autocomplete_1.autocompletion)(),
    (0, view_1.rectangularSelection)(),
    (0, view_1.crosshairCursor)(),
    (0, view_1.highlightActiveLine)(),
    (0, search_1.highlightSelectionMatches)(),
    view_1.keymap.of(tslib_1.__spreadArray(tslib_1.__spreadArray(tslib_1.__spreadArray(tslib_1.__spreadArray(tslib_1.__spreadArray(tslib_1.__spreadArray(tslib_1.__spreadArray([], autocomplete_1.closeBracketsKeymap, true), commands_1.defaultKeymap, true), search_1.searchKeymap, true), commands_1.historyKeymap, true), language_1.foldKeymap, true), autocomplete_1.completionKeymap, true), lint_1.lintKeymap, true))
];
var createCodeMirrorState = function (_a) {
    var onChange = _a.onChange, lock = _a.lock, dark = _a.dark, editable = _a.editable, value = _a.value;
    return state_1.EditorState.create({
        doc: value,
        extensions: [
            basicSetup,
            (0, lang_markdown_1.markdown)(),
            view_1.EditorView.updateListener.of(function (v) {
                if (v.focusChanged) {
                    lock.current = v.view.hasFocus;
                }
                if (v.docChanged) {
                    var getString = function () { return v.state.doc.toString(); };
                    onChange(getString);
                }
            }),
            view_1.EditorView.theme({
                '&.cm-focused': {
                    outline: 'none'
                }
            }, { dark: dark }),
            // https://github.com/codemirror/codemirror.next/issues/173
            view_1.EditorView.editable.of(editable)
        ]
    });
};
var createCodeMirrorView = function (_a) {
    var root = _a.root, options = tslib_1.__rest(_a, ["root"]);
    return new view_1.EditorView({
        state: createCodeMirrorState(options),
        parent: root
    });
};
exports.CodeMirror = react_1.default.forwardRef(function (_a, ref) {
    var value = _a.value, onChange = _a.onChange, lock = _a.lock, dark = _a.dark, editable = _a.editable;
    var divRef = react_1.default.useRef(null);
    var editorRef = react_1.default.useRef();
    var _b = react_1.default.useState(false), focus = _b[0], setFocus = _b[1];
    react_1.default.useEffect(function () {
        if (!divRef.current)
            return;
        var editor = createCodeMirrorView({
            root: divRef.current,
            onChange: onChange,
            lock: lock,
            dark: dark,
            editable: editable,
            value: value
        });
        editorRef.current = editor;
        return function () {
            editor.destroy();
        };
    }, [onChange, value, lock, dark]);
    react_1.default.useImperativeHandle(ref, function () { return ({
        update: function (value) {
            var current = editorRef.current;
            if (!current)
                return;
            current.setState(createCodeMirrorState({ onChange: onChange, lock: lock, dark: dark, editable: editable, value: value }));
        }
    }); });
    return (react_1.default.createElement("div", { onFocus: function () { return setFocus(true); }, onBlur: function () { return setFocus(false); }, className: style_module_css_1.default['code'] + (focus ? ' ' + style_module_css_1.default['focus'] : '') },
        react_1.default.createElement("div", { ref: divRef })));
});
//# sourceMappingURL=CodeMirror.js.map